; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version 04/2017																
; SUCHE nach unbekannten SLAVES im BUS											
; REGISTER:																		
;	temp	 Hilfsregister														
;	temp1	 Hilfsregister														
;	temp4	(4)Anzahl SLAVES													
;	temp6	(6)LastNull  (0...63)												
;	temp8	(8)NullFlag  (0...63)												
;	temp9	(9)BITzhler (0...63)												
;																				
; 	OUT: CARRY; SRAM(adr_ANZAHL_SLAVES), SRAM (adr_ROM_SLAVE_LISTE) "LISTE"		
;																				
ROM_SEARCH:
	rcall OW_ROM_LISTE_CLEAR
	rcall OW_ROM_CLEAR
	clr temp4							;(4)Anzahl SLAVES	
	ldi temp8,255						;(8)NullFlag		
;===============================================================================
ROM_SEARCH_NEXT_SLAVE:
	rcall OW_RESET_PRESENCE_C			;Reset und Presence-Test			
	brcs ROM_SEARCH_NEXT_SLAVE_w
	ret
ROM_SEARCH_NEXT_SLAVE_w:
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR); GRN aus			
	ldi	temp1,0xF0						; MASTER TX xF0 (SEARCH ROM)		
	rcall OW_BYTE_WRITE_TEMP1
	ldi	temp6,255						;(6)LastNnull		
	clr temp9							;(9)BITzhler		
;-------------------------------------------------------------------------------
ROM_SEARCH_NEXT_BIT:
	clr temp1
	rcall OW_BIT_READ_C					; BIT-A lesen		
	ROL temp1							; 1xlinks ber C	
	rcall OW_BIT_READ_C					; BIT-B lesen		
	ROL temp1							; 1xlinks ber C	
	cpi temp1,0b00000011				; ERROR (A)=(B)=1 ?	
	brne ROM_SEARCH_NEXT_BIT_w
	ldi temp,(ERROR_OW_SEARCH_AB)		; ERROR				
	STS(adr_ERROR_CODE),temp
	CLC									; CARRY clear		
	ret
;-------------------------------------------------------------------------------
ROM_SEARCH_NEXT_BIT_w:
	rcall ROM_SEARCH_AB_TEMP1			; vergleich (A)(B)	
	inc temp9							; alle Bits eingelesen ?
	cpi temp9,(63+1)					;(9)BITZHLER		
	brne ROM_SEARCH_NEXT_BIT			; ==> SCHLEIFE		
;===============================================================================
	rcall ROM_CRC7_C					; CRC-CHECK							
	brcs ROM_SEARCH_CRC_OK
	ldi temp,(ERROR_OW_SEARCH_CRC)
	STS(adr_ERROR_CODE),temp
	CLC									; CARRY clear		
	ret
ROM_SEARCH_CRC_OK:
;-------------------------------------------------------------------------------
	; max SLAVES						
	inc temp4							; ANZAHL SLAVES +1	
	cpi temp4,(ONE_WIRE_MAX_SLAVES+1)
	brlo ROM_SEARCH_OK
	ldi temp,(ERROR_OW_SEARCH_ANZAHL)	; ERROR				
	STS(adr_ERROR_CODE),temp
	CLC									; CARRY clear		
	ret
ROM_SEARCH_OK:
;-------------------------------------------------------------------------------
	rcall ROM_SEARCH_ERGEBNIS_SPEICHERN_TEMP4	; save		
	cpi temp6,255						;(6)LastNull		
	breq ROM_SEARCH_FERTIG
	mov temp8,temp6						;8)NullFlag save	
	rcall WAIT_10ms
	rjmp ROM_SEARCH_NEXT_SLAVE			; ==> SCHLEIFE		
;-------------------------------------------------------------------------------
ROM_SEARCH_FERTIG:
	STS(adr_ANZAHL_SLAVES),temp4		;(4)Anzahl SLAVES
	SEC	; CARRY setzen			
ret
; ##############################################################################
; inp: temp1																	
; A=Bit1=1	0b00000010															
; B=Bit0=1 	0b00000001															
; A=B=0 	0b00000000 (Diskrepanz)												
; A=B=1 	0b00000011 (Error)													
;																				
; A und B miteinander vergleichen (Diskrepanz)									
;																				
ROM_SEARCH_AB_TEMP1:
	tst temp1							; A=B=0 ?					
	breq ROM_SEARCH_DISK				; ja  =>Diskrepanz			
	SBRS temp1,1						; (A) zurcksenden (Bit1)	
	rjmp ROM_SEARCH_TX_NULL				; TX "0"					
	rjmp ROM_SEARCH_TX_EINS  			; TX "1"					
;-------------------------------------------------------------------------------
ROM_SEARCH_DISK:
	cp temp9,temp8						;(8)NullFlag vergleich		
	breq ROM_SEARCH_TX_EINS				; TX "1"					
	brlo ROM_SEARCH_DISK_w

	mov temp6,temp9						;(6)LastNull save			
	rjmp ROM_SEARCH_TX_NULL				; TX "0"					

ROM_SEARCH_DISK_w:
	LDS temp,(adr_ROM_ID+0)				; vorherige Antwort laden	
	ROR temp							; rechts, C	kullert raus	
	brcs ROM_SEARCH_TX_EINS				; TX "1"					
	mov temp6,temp9						;(6)LastNull save			
	rjmp ROM_SEARCH_TX_NULL				; TX "0"					
; ##############################################################################
; ##############################################################################
; ##############################################################################
; WRITE BIT	0																	
ROM_SEARCH_TX_NULL:
	CLC									; CARRY=0	
	rcall OW_BIT_WRITE_C				; INP: Carry
	CLC									; CARRY=0	
	rjmp ROM_SEARCH_BIT_SAVE_C			; INP: Carry
;-------------------------------------------------------------------------------
; WRITE BIT	1																	
ROM_SEARCH_TX_EINS:
	SEC									; CARRY=1	
	rcall OW_BIT_WRITE_C				; INP: Carry
	SEC									; CARRY=1	
	rjmp ROM_SEARCH_BIT_SAVE_C			; INP: Carry
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Schieberegister 64Bit															
; INP: Carry																	
ROM_SEARCH_BIT_SAVE_C:
	LDS temp,(adr_ROM_ID+7)	;
	ROR temp							; rechts ber C	
	STS(adr_ROM_ID+7),temp
	LDS temp,(adr_ROM_ID+6)	;
	ROR temp
	STS(adr_ROM_ID+6),temp
	LDS temp,(adr_ROM_ID+5)	;
	ROR temp
	STS(adr_ROM_ID+5),temp
	LDS temp,(adr_ROM_ID+4)	;
	ROR temp
	STS(adr_ROM_ID+4),temp
	LDS temp,(adr_ROM_ID+3)	;
	ROR temp
	STS(adr_ROM_ID+3),temp
	LDS temp,(adr_ROM_ID+2)	;
	ROR temp
	STS(adr_ROM_ID+2),temp
	LDS temp,(adr_ROM_ID+1)	;
	ROR temp
	STS(adr_ROM_ID+1),temp
	LDS temp,(adr_ROM_ID+0)	;
	ROR temp
	STS(adr_ROM_ID+0),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ROM wird im SRAM abgelegt														
;																				
ROM_SEARCH_ERGEBNIS_SPEICHERN_TEMP4:
	ldi ZL,LOW (adr_ROM_LISTE)	; Z-ZEIGER					
	ldi ZH,HIGH(adr_ROM_LISTE)
	ldi XL,LOW (adr_ROM_ID)		; X-ZEIGER					
	ldi XH,HIGH(adr_ROM_ID)
	mov temp1,temp4				;(4)Anzahl SLAVES
	dec temp1					; -1	
	ldi temp2,8					; mal 8						
	mul temp1,temp2
	add ZL,R0
	adc ZH,R1

	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
	LD temp,X+		; 
	ST Z+,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
